<?php

declare(strict_types=1);

namespace Drupal\commerce_coinspaid\PluginForm\OffsiteRedirect;

use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_coinspaid\Service\CoinspaidService;
use Drupal\commerce_coinspaid\Service\DebugService;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;

class CoinspaidOffsiteForm extends BasePaymentOffsiteForm
{
    use MessengerTrait;

  /**
   * @param array $form
   * @param FormStateInterface $form_state
   *
   * @return array
   * @throws \Drupal\commerce\Response\NeedsRedirectException
   * @throws \Exception
   */
    public function buildConfigurationForm(
        array $form,
        FormStateInterface $form_state
    ): array {
        $form = parent::buildConfigurationForm($form, $form_state);
        $payment = $this->getEntity();
        $order_id = \Drupal::routeMatch()->getParameter('commerce_order')->id();
        $order = Order::load($order_id);
        $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

        $plugin_configuration = $payment_gateway_plugin->getConfiguration();
        DebugService::setEnabled((bool) $plugin_configuration['debug']);

        $this->messenger()->addMessage($this->t('Payment failed. Please check information and try again.'), 'error');

        $coinspaid_service = new CoinspaidService(
            $plugin_configuration['public_key'],
            $plugin_configuration['secret_key'],
            $plugin_configuration['mode']
        );


        $coinspaid_service->createInvoice($this->prepareOrder($order, $form, $plugin_configuration));
        $redirect_url = $coinspaid_service->getRedirectLink();
        if (empty($redirect_url)) {
            if ($coinspaid_service->hasErrors()) {
                foreach ($coinspaid_service->getErrors() as $error_message) {
                    if (is_string($error_message)) {
                        \Drupal::messenger()->addMessage($error_message);
                    }
                }
            }
            throw new NeedsRedirectException($form['#cancel_url']);
        }

        return $this->buildRedirectForm($form, $form_state, $redirect_url, []);
    }

  /**
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   * @param array $form
   * @param array $plugin_configuration
   *
   * @return array
   */
    private function prepareOrder(OrderInterface $order, array $form, array $plugin_configuration): array
    {
        $titles = [];
        foreach ($order->getItems() as $orderItem) {
            $titles[] = $orderItem->getTitle();
        }
      /* @var \Drupal\commerce_price\Price $amount */
        $amount = $this->getEntity()->getAmount();

        return [
        'timer' => filter_var($plugin_configuration['invoice_mode'], FILTER_VALIDATE_BOOLEAN),
        'title' => $order->id(),
        'currency' => $amount->getCurrencyCode(),
        'amount' => $amount->getNumber(),
        'foreign_id' => $order->id() . '-' . time(),
        'url_success' => $form['#return_url'],
        'url_failed' => $form['#cancel_url'],
        'email_user' => $order->getEmail(),
        'description' => implode(';', $titles),
        ];
    }
}
